@@ -4,10 +4,9 @@ use crate::parser::key::key;
 use crate::parser::table::duplicate_key;
 use crate::parser::trivia::ws;
 use crate::parser::value::value;
-use crate::repr::InternalString;
 use crate::table::TableKeyValue;
-use crate::{InlineTable, Item, Value};
-use combine::parser::char::char;
+use crate::{InlineTable, InternalString, Item, Value};
+use combine::parser::byte::byte;
 use combine::stream::RangeStream;
 use combine::*;
 
@@ -15,7 +14,7 @@ use combine::*;
 
 // inline-table = inline-table-open inline-table-keyvals inline-table-close
 parse!(inline_table() -> InlineTable, {
-    between(char(INLINE_TABLE_OPEN), char(INLINE_TABLE_CLOSE),
+    between(byte(INLINE_TABLE_OPEN), byte(INLINE_TABLE_CLOSE),
             inline_table_keyvals().and_then(|(kv, p)| table_from_pairs(kv, p)))
 });
 
@@ -25,26 +24,29 @@ fn table_from_pairs(
 ) -> Result<InlineTable, CustomError> {
     let mut root = InlineTable::new();
     root.preamble = InternalString::from(preamble);
+    // Assuming almost all pairs will be directly in `root`
+    root.items.reserve(v.len());
 
     for (path, kv) in v {
-        let table = descend_path(&mut root, &path, 0)?;
-        if table.contains_key(kv.key.get()) {
+        let table = descend_path(&mut root, &path)?;
+        let key: InternalString = kv.key.get_internal().into();
+        let old = table.items.insert(key.clone(), kv);
+        let duplicate_key = old.is_some();
+        if duplicate_key {
             return Err(CustomError::DuplicateKey {
-                key: kv.key.into(),
+                key: key.as_str().into(),
                 table: "inline".into(),
             });
         }
-        table.items.insert(kv.key.get().to_owned(), kv);
     }
     Ok(root)
 }
 
 fn descend_path<'a>(
-    table: &'a mut InlineTable,
+    mut table: &'a mut InlineTable,
     path: &'a [Key],
-    i: usize,
 ) -> Result<&'a mut InlineTable, CustomError> {
-    if let Some(key) = path.get(i) {
+    for (i, key) in path.iter().enumerate() {
         let entry = table.entry_format(key).or_insert_with(|| {
             let mut new_table = InlineTable::new();
             new_table.set_dotted(true);
@@ -53,23 +55,24 @@ fn descend_path<'a>(
         });
         match *entry {
             Value::InlineTable(ref mut sweet_child_of_mine) => {
-                descend_path(sweet_child_of_mine, path, i + 1)
+                table = sweet_child_of_mine;
+            }
+            _ => {
+                return Err(duplicate_key(path, i));
             }
-            _ => Err(duplicate_key(path, i)),
         }
-    } else {
-        Ok(table)
     }
+    Ok(table)
 }
 
 // inline-table-open  = %x7B ws     ; {
-const INLINE_TABLE_OPEN: char = '{';
+pub(crate) const INLINE_TABLE_OPEN: u8 = b'{';
 // inline-table-close = ws %x7D     ; }
-const INLINE_TABLE_CLOSE: char = '}';
+const INLINE_TABLE_CLOSE: u8 = b'}';
 // inline-table-sep   = ws %x2C ws  ; , Comma
-const INLINE_TABLE_SEP: char = ',';
+const INLINE_TABLE_SEP: u8 = b',';
 // keyval-sep = ws %x3D ws ; =
-pub(crate) const KEYVAL_SEP: char = '=';
+pub(crate) const KEYVAL_SEP: u8 = b'=';
 
 // inline-table-keyvals = [ inline-table-keyvals-non-empty ]
 // inline-table-keyvals-non-empty =
@@ -78,7 +81,7 @@ pub(crate) const KEYVAL_SEP: char = '=';
 
 parse!(inline_table_keyvals() -> (Vec<(Vec<Key>, TableKeyValue)>, &'a str), {
     (
-        sep_by(keyval(), char(INLINE_TABLE_SEP)),
+        sep_by(keyval(), byte(INLINE_TABLE_SEP)),
         ws(),
     )
 });
@@ -86,11 +89,11 @@ parse!(inline_table_keyvals() -> (Vec<(Vec<Key>, TableKeyValue)>, &'a str), {
 parse!(keyval() -> (Vec<Key>, TableKeyValue), {
     (
         key(),
-        char(KEYVAL_SEP),
+        byte(KEYVAL_SEP),
         (ws(), value(), ws()),
     ).map(|(key, _, v)| {
-            let mut path = key.into_vec();
-            let key = path.pop().expect("Was vec1, so at least one exists");
+        let mut path = key;
+        let key = path.pop().expect("grammar ensures at least 1");
 
         let (pre, v, suf) = v;
         let v = v.decorated(pre, suf);
